@@ -1,6 +1,8 @@
 use error::Error;
 use std::process::{Command, Stdio};
 
+use self::error::{RemoveContainerError, RunContainerError, StopContainerError};
+
 pub fn setup(podman: bool) -> Result<(), Error> {
     spawn_container(podman)?;
     healthcheck(podman, 120, 1000)?;
@@ -13,9 +15,9 @@ pub fn cleanup(podman: bool) -> Result<(), Error> {
     Ok(())
 }
 
-fn spawn_container(podman: bool) -> Result<(), Error> {
+fn spawn_container(podman: bool) -> Result<(), RunContainerError> {
     let command = if podman { "podman" } else { "docker" };
-    Command::new(&command)
+    let success = Command::new(&command)
         .arg("run")
         .arg("-d")
         .arg("--name")
@@ -27,11 +29,14 @@ fn spawn_container(podman: bool) -> Result<(), Error> {
         .arg("postgres")
         .stderr(Stdio::null())
         .stdout(Stdio::null())
-        .spawn()
-        .map_err(Error::RunContainer)?
-        .wait()
-        .map_err(Error::RunContainer)?;
-    Ok(())
+        .status()?
+        .success();
+
+    if success {
+        Ok(())
+    } else {
+        Err(RunContainerError::Status)
+    }
 }
 
 fn is_postgres_healthy(podman: bool) -> Result<bool, Error> {
@@ -65,33 +70,39 @@ fn healthcheck(podman: bool, max_retries: u64, ms_per_retry: u64) -> Result<(),
     Ok(())
 }
 
-fn stop_container(podman: bool) -> Result<(), Error> {
+fn stop_container(podman: bool) -> Result<(), StopContainerError> {
     let command = if podman { "podman" } else { "docker" };
-    Command::new(&command)
+    let success = Command::new(&command)
         .arg("stop")
         .arg("cornucopia_postgres")
         .stderr(Stdio::null())
         .stdout(Stdio::null())
-        .spawn()
-        .map_err(Error::StopContainer)?
-        .wait()
-        .map_err(Error::StopContainer)?;
+        .status()?
+        .success();
 
-    Ok(())
+    if success {
+        Ok(())
+    } else {
+        Err(StopContainerError::Status)
+    }
 }
 
-fn remove_container(podman: bool) -> Result<(), Error> {
+fn remove_container(podman: bool) -> Result<(), RemoveContainerError> {
     let command = if podman { "podman" } else { "docker" };
-    Command::new(&command)
+    let success = Command::new(&command)
         .arg("rm")
         .arg("-v")
         .arg("cornucopia_postgres")
         .stderr(Stdio::null())
         .stdout(Stdio::null())
-        .spawn()
-        .map_err(Error::RemoveContainer)?;
+        .status()?
+        .success();
 
-    Ok(())
+    if success {
+        Ok(())
+    } else {
+        Err(RemoveContainerError::Status)
+    }
 }
 
 pub mod error {
@@ -101,14 +112,38 @@ pub mod error {
     #[error("Error encountered while running docker command. Please check that docker is installed, and that the daemon is running. If you are a Linux user, please check that you are in the `docker` group")]
     pub enum Error {
         #[error("couldn't start database container")]
-        RunContainer(std::io::Error),
+        RunContainer(#[from] RunContainerError),
         #[error("encountered error while probing database container health")]
         HealthCheck(std::io::Error),
         #[error("couldn't stop database container")]
-        StopContainer(std::io::Error),
+        StopContainer(#[from] StopContainerError),
         #[error("couldn't clean up database container")]
-        RemoveContainer(std::io::Error),
+        RemoveContainer(#[from] RemoveContainerError),
         #[error("max number of retries reached while waiting for database container to start")]
         MaxNbRetries,
     }
+
+    #[derive(Debug, ThisError)]
+    #[error("couldn't start database container")]
+    pub enum RunContainerError {
+        Io(#[from] std::io::Error),
+        #[error("command returned with an error status")]
+        Status,
+    }
+
+    #[derive(Debug, ThisError)]
+    #[error("couldn't stop database container")]
+    pub enum StopContainerError {
+        Io(#[from] std::io::Error),
+        #[error("command returned with an error status")]
+        Status,
+    }
+
+    #[derive(Debug, ThisError)]
+    #[error("couldn't clean up database container")]
+    pub enum RemoveContainerError {
+        Io(#[from] std::io::Error),
+        #[error("command returned with an error status")]
+        Status,
+    }
 }
